package org.jobimtext.examples.api;

import java.util.Arrays;
import java.util.List;

import org.jobimtext.api.struct.DatabaseThesaurusDatastructure;
import org.jobimtext.api.struct.IThesaurusDatastructure;
import org.jobimtext.api.struct.Order2;
import org.jobimtext.api.struct.Sense;

public class MySqlExample {
	IThesaurusDatastructure<String, String> dt ;
	public void init(String config ){
		dt= new DatabaseThesaurusDatastructure(config);
		dt.connect();
		
	}
	public void destroy(){
		dt.destroy();
	}
	public static void main(String[] args) {
		
		System.out.println("---------------------------------------");
		System.out.println("---------------------------------------");
		System.out.println("Trigram Model");
		System.out.println("---------------------------------------");
		String config = "conf_mysql_de_70M_trigram.xml";
		MySqlExample examples= new MySqlExample();
		examples.init(config);
		String term = "Maus";
		examples.outputSimilarTerms(term);
		examples.outputContextsForTerm(term);
		examples.outputCounts(term);
		examples.outputSenses(term);
		examples.outputSimilarTermsFromSentence("Die Katze frisst die Maus");
		examples.destroy();
		
		System.out.println("---------------------------------------");
		System.out.println("---------------------------------------");
		System.out.println("Mate Parser Model");
		System.out.println("---------------------------------------");
		config = "conf_mysql_de_70M_mate.xml";
		examples.init(config);
		term = "Maus";
		examples.outputSimilarTerms(term);
		examples.outputContextsForTerm(term);
		examples.outputCounts(term);
		examples.outputSenses(term);
		examples.outputSimilarTermsFromSentence("Die Katze frisst die Maus");
		examples.destroy();
		
	}
	
	/**
	 * Output of the frequency of a term and the frequency of a context feature of that term
	 * @param term
	 */
	private void outputCounts(String term) {
		System.out.println("--------------------------------------------------------------------");
		System.out.println("Term count: ");
		System.out.println(term +"\t"+dt.getTermCount(term));
		System.out.println("Context count:");
		String context = dt.getTermContextsScores(term,1).get(0).key;
		System.out.println(context+"\t"+dt.getContextsCount(context));
		System.out.println();
	}
	private void outputSenses(String term) {
		System.out.println("--------------------------------------------------------------------");
		System.out.println("Available sense clusters:");
		String[] senses = dt.getSensesTypes();
		System.out.println(Arrays.toString(senses));
		System.out.println();
		for (String sense: senses){
			String append = "";
			if(sense.equals(dt.getStandardSense())){
				append = " [standard sense]";
			}
			System.out.println("Output for sense [sense is, sense words, ISA labels]: "+sense+append);
			for (Sense s: dt.getSenses(term, sense)){
				System.out.println(s);
			}
			System.out.println();
			
		}
		System.out.println();
	}
	private void outputSimilarTerms(String term){
		System.out.println("--------------------------------------------------------------------");
		System.out.println("Output of all similar terms for "+term);
		System.out.println(dt.getSimilarTerms(term));
		System.out.println();
		System.out.println("Output of the 3 most similar terms for "+term);
		System.out.println(dt.getSimilarTerms(term,3));
		System.out.println();
		System.out.println("Output of the all similar terms with a similarity score above 20 for "+term);
		System.out.println(dt.getSimilarTerms(term,81.0));
		System.out.println();
	}
	
	private void outputContextsForTerm(String term){
		System.out.println("--------------------------------------------------------------------");
		System.out.println("Output of all contexts features for term: "+term);
		System.out.println(dt.getTermContextsScores(term));
		System.out.println();
		System.out.println("Output of the 3 highest ranked context features for the term: "+term);
		System.out.println(dt.getTermContextsScores(term,3));
		System.out.println();
		System.out.println("Output of the all context features with a score above 100.0 for the term: "+term);
		System.out.println(dt.getTermContextsScores(term,100.0));
		System.out.println();
	}

	
	private void outputSimilarTermsFromSentence(String sentence){
		System.out.println("--------------------------------------------------------------------");
		System.out.println("Print similar terms for whitespace separated sentence:");
		System.out.println(sentence);
		System.out.println();
		for(String token: sentence.split(" ")){
			List<Order2> similarities = dt.getSimilarTerms(token,10);
			System.out.print(token+":");
			for (Order2 o2 : similarities){
				System.out.print("\t"+o2.key+":"+o2.score+"\t");
			}
			System.out.println();
		}
		System.out.println();
		
	}
	
	
	
}
