package org.jobimtext.examples.api;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.pig.impl.util.Pair;
import org.jobimtext.api.struct.Sense;
import org.jobimtext.api.struct.WebThesaurusDatastructure;
import org.jobimtext.api.struct.WebThesaurusInterface;


public class WebApiContextualizationExample {
	
	static WebThesaurusInterface<String, String> dtWeb;
	
	public static void main(String[] args) {
		
		
		dtWeb = new WebThesaurusDatastructure("conf_web_wikipedia_trigram.xml");		
		dtWeb.connect();
		
		// simple sentences for contextualization
		String sentence1 = "The mouse button is stuck";
		String sentence2 = "They analyzed human and mouse genomes";
		
		String target =  "mouse";
		
		
		System.out.println("Sentence 1: "+sentence1);
		List<Pair<String, List<String>>> holing1 = dtWeb.getSentenceHoling(sentence1);		
		contextualize(target, holing1);
		
		
		System.out.println("\n\nSentence 2: "+sentence2);
		List<Pair<String, List<String>>> holing2 = dtWeb.getSentenceHoling(sentence2);
		contextualize(target, holing2);
		
		
	}

	/**
	 * Simple contextualization method for a single target word, just using the Bim from the sentence
	 * @param target the word that should be contextualized
	 * @param holing the result of a sentence holing operation
	 */
	private static void contextualize(String target, List<Pair<String, List<String>>> holing) {
		System.out.println("\nContextualization for the term '"+target+"'");
		
		for (Pair<String, List<String>> p : holing) {
			// identify target
			if (p.first.compareTo(target) == 0) {

				// get Bim (trigram holing only returns one Bim per term)
				String bim = p.second.get(0);
				System.out.println("Bim used for contextualization: "+bim);

				// get all senses for target term
				List<Sense> senses = dtWeb.getSenses(target);
				int senseId = 0;
				
				// results
				Map<Integer, Long> counts = new HashMap<Integer, Long>();
				
				System.out.println("SenseID\tTerms");
				// process all senses
				for (Sense s : senses) {
					// check all sense terms
					for (String term : s.getSenses()) {
						// if there is a match between sense-term and Bim, add the count to the results
						Long count =  dtWeb.getTermContextsCount(term, bim);
						if (count > 0) {
							if (counts.containsKey(senseId)) {
								counts.put(senseId, counts.get(senseId)+count);
							} else {
								counts.put(senseId, count);
							}
						}
					}
					System.out.println(senseId+"\t"+s.getSenses());
					senseId++;
				}
				System.out.println("\nMatches for following senses:\t"+counts);
			}
		}
	}

}
