package org.jobimtext.examples.api;

import org.jobimtext.api.struct.IThesaurusDatastructure;
import org.jobimtext.api.struct.WebThesaurusDatastructure;
import org.jobimtext.api.struct.WebThesaurusInterface;


public class WebApiExample {
	public static void main(String[] args) {
		IThesaurusDatastructure<String, String> dt;		
		dt = new WebThesaurusDatastructure("conf_web_wikipedia_trigram.xml");
		dt.connect();
		
		WebThesaurusInterface<String, String> dtWeb;
		dtWeb = new WebThesaurusDatastructure("conf_web_wikipedia_trigram.xml");		
		dtWeb.connect();
		
		System.out.println("Sentence Holing:\t"+dtWeb.getSentenceHoling("this is a test sentence"));
		System.out.println();
		
		String term1 = "test";
		String term2 = "tests";
		String context = "Bim_@_Bim(a_@_driver)";
		
		// similar terms
		System.out.println("Similar terms for '"+term1+"':\t"+dt.getSimilarTerms(term1));
		System.out.println("Similar terms for '"+term1+"' (top 5):\t"+dt.getSimilarTerms(term1, 5));
		System.out.println("Similar terms for '"+term1+"' (score threshold):\t"+dt.getSimilarTerms(term1, 100.0));		
		
		System.out.println("Similarity score between '"+term1+"' and '"+term2+"':\t"+dt.getSimilarTermScore(term1, term2));
		System.out.println();
		
		// counts
		System.out.println("Term count for '"+term1+"':\t"+dt.getTermCount(term1));
		System.out.println("Context count for '"+context+"':\t"+dt.getContextsCount(context));
		System.out.println();
		
		// term context scores
		System.out.println("Context scores for '"+term1+"':\t"+dt.getTermContextsScores(term1));
		System.out.println("Context scores for '"+term1+"' (top 3):\t"+dt.getTermContextsScores(term1, 3));
		System.out.println("Context scores for '"+term1+"' (score threshold):\t"+dt.getTermContextsScores(term1, 5000.0));
		
		System.out.println("Term-Context count for '"+term1+"' & '"+context+"':\t"+dt.getTermContextsCount(term1, context));
		System.out.println("Term-Context score for '"+term1+"' & '"+context+"':\t"+dt.getTermContextsScore(term1, context));
		System.out.println();
		
		// context term scores
		System.out.println("Term scores for '"+context+"':\t"+dt.getContextsTermScores(context));
		System.out.println("Term scores for '"+context+"' (top 3):\t"+dt.getContextsTermScores(context, 3));
		System.out.println("Term scores for '"+context+"' (score threshold):\t"+dt.getContextsTermScores(context, 500.0));
		System.out.println();
		
		// sense types
		String[] types = dt.getSensesTypes();
		System.out.print("Available Sense types:\t");
		for (String s : types) {
			System.out.print(s+"\t");
		}
		System.out.println();
		System.out.println("Standard Sense type:\t"+dt.getStandardSense());
		System.out.println();
		
		// senses, isas for standard sense (same output for all three methods)
		System.out.println("Standard senses:\t"+dt.getSenses(term1));
		System.out.println("Standard ISAs:\t\t"+dt.getIsas(term1));
		System.out.println("Standard sense ids:\t"+dt.getSenseCUIs(term1));
				
		System.out.println("Senses for the 'CW-finer' type:\t\t"+dt.getSenses(term1, "CW-finer"));
		System.out.println("ISAs for the 'CW-finer' type:\t\t"+dt.getIsas(term1, "CW-finer"));
		System.out.println("Sense ids for the 'CW-finer' type:\t"+dt.getSenseCUIs(term1, "CW-finer"));
		System.out.println();
		
		System.out.println("Is '"+term1+"' contained in the corpus?\t"+dt.isTermContained(term1));
		System.out.println("Is 'notcontained' contained in the corpus?\t"+dt.isTermContained("notcontained"));
		
		System.out.println("Which of the following terms are contained in the corpus: test, contain, notcontained, new?"); 
		String[] contain = dt.isTermContained("test", "contain", "notcontained", "new");
		for (String s: contain) {
			System.out.print(s+"\t");
		}
		
	}

}
