package org.jobimtext.api.db;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.hadoop.util.StringUtils;
import org.jobimtext.api.IThesaurus;
import org.jobimtext.api.configuration.SenseTables;

public class DatabaseThesaurus extends DatabaseResource
		implements
		IThesaurus<String, String, ResultSet, ResultSet, ResultSet, ResultSet, ResultSet> {

	private static final String DEFAULT_SENSE = "DEFAULT_SENSE";
	private HashMap<String, SenseTables> sensesMap = null;

	@Override
	public ResultSet getSimilarTerms(String key) {
		String sql = getDatabaseConfiguration().getSimilarTermsQuery();
		return getResultSetFromStringArgument(key, sql);

	}

	@Override
	public ResultSet getSimilarTerms(String key, int numberOfEntries) {
		String sql = getDatabaseConfiguration().getSimilarTermsTopQuery(
				numberOfEntries);
		return getResultSetFromStringArgument(key, sql);

	}

	@Override
	public ResultSet getSimilarTerms(String key, double threshold) {
		String sql = getDatabaseConfiguration().getSimilarTermsGtScoreQuery();

		try {
			PreparedStatement ps = getDatabaseConnection().getConnection()
					.prepareStatement(sql);
			ps.setString(1, key);
			ps.setDouble(2, threshold);
			return ps.executeQuery();
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return null;
	}

	@Override
	public ResultSet getSimilarContexts(String values) {
		String sql = getDatabaseConfiguration().getSimilarContextsQuery();
		return getResultSetFromStringArgument(values, sql);

	}

	@Override
	public ResultSet getSimilarContexts(String values, int numberOfEntries) {
		String sql = getDatabaseConfiguration().getSimilarContextsTopQuery(
				numberOfEntries);
		return getResultSetFromStringArgument(values, sql);

	}

	@Override
	public ResultSet getSimilarContexts(String values, double threshold) {
		String sql = getDatabaseConfiguration()
				.getSimilarContextsGtScoreQuery();

		try {

			PreparedStatement ps = getDatabaseConnection().getConnection()
					.prepareStatement(sql);
			ps.setString(1, values);
			ps.setDouble(2, threshold);

			return ps.executeQuery();
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return null;
	}

	@Override
	public Long getTermCount(String key) {
		Long count = 0L;
		String sql = getDatabaseConfiguration().getTermsCountQuery();
		PreparedStatement ps;
		try {
			ps = getDatabaseConnection().getConnection().prepareStatement(sql);

			ps.setString(1, key);
			ResultSet set = ps.executeQuery();

			if (set.next()) {
				count = set.getLong(1);
			}
			ps.close();
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return count;
	}

	@Override
	public Long getContextsCount(String values) {
		Long count = 0L;
		String sql = getDatabaseConfiguration().getContextsCountQuery();
		PreparedStatement ps;
		try {
			ps = getDatabaseConnection().getConnection().prepareStatement(sql);
			ps.setString(1, values);
			ResultSet set = ps.executeQuery();
			if (set.next()) {
				count = set.getLong(1);
			}
			ps.close();
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return count;
	}

	@Override
	public Long getTermContextsCount(String key, String values) {
		Long count = 0L;
		String sql = getDatabaseConfiguration().getTermContextsCountQuery();
		PreparedStatement ps;
		try {
			ps = getDatabaseConnection().getConnection().prepareStatement(sql);
			ps.setString(1, key);
			ps.setString(2, values);
			ResultSet set = ps.executeQuery();
			if (set.next()) {
				count = set.getLong(1);
			}
			ps.close();
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return count;
	}

	@Override
	public Double getTermContextsScore(String key, String val) {
		double score = 0.0;
		String sql = getDatabaseConfiguration().getTermContextsScoreQuery();
		PreparedStatement ps;
		try {
			ps = getDatabaseConnection().getConnection().prepareStatement(sql);
			ps.setString(1, key);
			ps.setString(2, val);
			ResultSet set = ps.executeQuery();
			if (set.next()) {
				score = set.getDouble(1);

			}
			ps.close();
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return score;
	}

	@Override
	public ResultSet getTermContextsScores(String key) {
		String sql = getDatabaseConfiguration().getTermContextsScoresQuery();
		return getResultSetFromStringArgument(key, sql);

	}

	@Override
	public ResultSet getTermContextsScores(String key, int numberOfEntries) {
		String sql;
		sql = getDatabaseConfiguration().getTermContextsScoresTopQuery(
				numberOfEntries);
		return getResultSetFromStringArgument(key, sql);

	}

	@Override
	public ResultSet getTermContextsScores(String key, double threshold) {
		String sql = getDatabaseConfiguration().getTermContextsScoresGtScore();
		PreparedStatement ps;
		try {
			ps = getDatabaseConnection().getConnection().prepareStatement(sql);
			ps.setString(1, key);
			ps.setDouble(2, threshold);
			ResultSet set = ps.executeQuery();

			return set;
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return null;
	}

	// @Override
	// public ResultSet getSenses(String key) {
	// String sql = getDatabaseConfiguration().getSensesQuery();
	// PreparedStatement ps;
	// try {
	// ps = getDatabaseConnection().getConnection().prepareStatement(sql);
	// ps.setString(1, key);
	// ResultSet set = ps.executeQuery();
	//
	// return set;
	// } catch (SQLException e) {
	// System.err.println(e.getMessage());
	// System.err.println("Query: "+sql);
	// }
	// return null;
	// }

	// @Override
	// public ResultSet getIsas(String key) {
	// String sql = getDatabaseConfiguration().getIsasQuery();
	// return getResultSetFromOneStringArgument(key, sql);
	//
	// }
	private ResultSet getResultSetFromStringArgument(String value, String sql) {
		PreparedStatement ps;
		try {
			ps = getDatabaseConnection().getConnection().prepareStatement(sql);
			ps.setString(1, value);
			ResultSet set = ps.executeQuery();

			return set;
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return null;
	}

	private ResultSet getResultSetFromStringStringArgument(String value1,
			String value2, String sql) {
		PreparedStatement ps;
		try {
			ps = getDatabaseConnection().getConnection().prepareStatement(sql);
			ps.setString(1, value1);
			ps.setString(2, value2);
			ResultSet set = ps.executeQuery();

			return set;
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return null;
	}

	// @Override
	// public ResultSet getSenseCUIs(String key) {
	// String sql = getDatabaseConfiguration().getSensesCUIsQuery();
	// return getResultSetFromOneStringArgument(key, sql);
	// }

	@Override
	public Double getSimilarTermScore(String t1, String t2) {
		double score = 0.0;
		String sql = getDatabaseConfiguration().getSimilarTermScoreQuery();
		PreparedStatement ps;
		try {
			ps = getDatabaseConnection().getConnection().prepareStatement(sql);
			ps.setString(1, t1);
			ps.setString(2, t2);
			ResultSet set = ps.executeQuery();
			if (set.next()) {
				score = set.getDouble(1);

			}
			ps.close();
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return score;
	}

	@Override
	public ResultSet getContextsTermScores(String key) {
		String sql = getDatabaseConfiguration().getContextsTermScoresQuery();
		PreparedStatement ps;
		try {
			ps = getDatabaseConnection().getConnection().prepareStatement(sql);
			ps.setString(1, key);
			ResultSet set = ps.executeQuery();

			return set;
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return null;
	}

	@Override
	public ResultSet getContextsTermScores(String key, int numberOfEntries) {
		String sql = getDatabaseConfiguration().getContextsTermScoresQueryTopN(
				numberOfEntries);
		PreparedStatement ps;
		try {
			ps = getDatabaseConnection().getConnection().prepareStatement(sql);
			ps.setString(1, key);
			ResultSet set = ps.executeQuery();

			return set;
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return null;
	}

	@Override
	public ResultSet getContextsTermScores(String key, double threshold) {
		String sql = getDatabaseConfiguration()
				.getContextsTermScoresQueryThreshold();
		PreparedStatement ps;
		try {
			ps = getDatabaseConnection().getConnection().prepareStatement(sql);
			ps.setString(1, key);
			ps.setDouble(2, threshold);

			ResultSet set = ps.executeQuery();

			return set;
		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		return null;
	}

	@Override
	public ResultSet getSenses(String key) {
		String sql = getDatabaseConfiguration().getSensesFromDbQuery(
				getSensesTable(getStandardSense()).getTableSenses());
		return getResultSetFromStringArgument(key, sql);
	}

	@Override
	public ResultSet getIsas(String key) {
		String sql = getDatabaseConfiguration().getIsasFromDbQuery(
				getSensesTable(getStandardSense()).getTableIsas());
		return getResultSetFromStringArgument(key, sql);

	}

	@Override
	public ResultSet getSenseCUIs(String key) {
		String sql = getDatabaseConfiguration().getSensesCUIsFromDbQuery(
				getSensesTable(getStandardSense()).getTableSenseCUIs());
		return getResultSetFromStringArgument(key, sql);

	}

	@Override
	public ResultSet getSenses(String key, String type) {
		String sql = getDatabaseConfiguration().getSensesFromDbQuery(
				getSensesTable(type).getTableSenses());
		return getResultSetFromStringArgument(key, sql);
	}

	@Override
	public ResultSet getIsas(String key, String type) {
		String sql = getDatabaseConfiguration().getIsasFromDbQuery(
				getSensesTable(type).getTableIsas());
		return getResultSetFromStringArgument(key, sql);

	}

	@Override
	public ResultSet getSenseCUIs(String key, String type) {
		String sql = getDatabaseConfiguration().getSensesCUIsFromDbQuery(
				getSensesTable(type).getTableSenseCUIs());
		return getResultSetFromStringArgument(key, sql);

	}

	private SenseTables getSensesTable(String type) {
		createSenseMap();
		if(sensesMap==null)return null;
		return sensesMap.get(type);
	}

	private void createSenseMap() {
		if (getDatabaseConfiguration().getTables().getSenses() == null) {
			System.err
					.println("There are no sense tables in the database configuration file");
			return;
		}
		if (sensesMap == null) {
			sensesMap = new HashMap<String, SenseTables>();

			for (SenseTables s : getDatabaseConfiguration().getTables()
					.getSenses()) {
				sensesMap.put(s.getName(), s);
				if (s.isDefault()) {
					sensesMap.put(DEFAULT_SENSE, s);
				}
			}
		}
	}

	@Override
	public boolean isTermContained(String key) {
		if (isTermContained(new String[] { key }).length == 1) {
			return true;
		}
		return false;
	}

	public String[] isXContained(String sql,String... key) {
		int numArguments = sql.length() - sql.replace("?", "").length();
		Set<String> list = new HashSet<String>();
		int j = 0;
		int iterations = key.length / numArguments;
		if (key.length % numArguments != 0) {
			iterations++;
		}
		PreparedStatement ps;
		try {

			for (int i = 0; i < iterations; i++) {
				ps = getDatabaseConnection().getConnection().prepareStatement(
						sql);
				for (int k = 0; k < numArguments; k++, j++) {
					if (j >= key.length) {
						// jump back to the actual round, so no additional
						// computation time is neccessary as it's the same word
						// which is already in there
						j = j - j % numArguments;
					}
					ps.setString(k + 1, key[j]);

				}
				ResultSet set = ps.executeQuery();
				while (set.next()) {
					list.add(set.getString(1));
				}
			}

		} catch (SQLException e) {
			System.err.println(e.getMessage());
			System.err.println("Query: " + sql);
		}
		String[] values = new String[list.size()];
		int i = 0;
		for (String s : list) {
			values[i] = s;
			i++;
		}
		return values;

		
	}
	
	@Override
	public String[] isTermContained(String... key) {
		String sql = getDatabaseConfiguration().isTermContained(key);
		return isXContained(sql, key);
	}

	@Override
	public String[] getSensesTypes() {

		List<SenseTables> s = this.getDatabaseConfiguration().getTables()
				.getSenses();
		String[] senses = new String[s.size()];
		int i = 0;
		for (SenseTables t : s) {
			senses[i++] = t.getName();
		}
		return senses;
	}

	@Override
	public String getStandardSense() {
		createSenseMap();
		
		return sensesMap.get(DEFAULT_SENSE).getName();
	}

	@Override
	public boolean isContextsContained(String key) {
		String[] result = isContextsContained(new String[]{key});
		if(result.length>0)
			return true;
		return false;
				
	}

	@Override
	public String[] isContextsContained(String... key) {
		String sql = getDatabaseConfiguration().isContextsContained(key);
		return isXContained(sql, key);
	}
}
