package org.jobimtext.api.map;

/*******************************************************************************
 * Copyright 2012
 * Copyright (c) 2012 IBM Corp.
 * 
 * and
 * 
 * FG Sprachtechnologie / Language Technology
 * Technische Universitaet Darmstadt
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
import java.io.File;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.pig.impl.util.Pair;
import org.jobimtext.api.configuration.DatabaseThesaurusConfiguration;
import org.jobimtext.api.db.DatabaseResource;
import org.jobimtext.api.db.DatabaseThesaurus;
import org.jobimtext.api.struct.Sense;
import org.jobimtext.api.webthesaurus.WebThesaurus;

public class WebThesaurusMap implements
		IThesaurusMap<String, String> {

	WebThesaurus webThesaurus = new WebThesaurus();

	public WebThesaurusMap(File dbConfigurationFile) {
		webThesaurus.setWebConfigurationFile(dbConfigurationFile);
	}

	public WebThesaurusMap(String dbConfigurationFile) {
		this(new File(dbConfigurationFile));
	}

	public WebThesaurusMap(File dbConfigurationFile,
			File tunnelConfigurationFile) {
		webThesaurus.setWebConfigurationFile(dbConfigurationFile);
	}

	public WebThesaurusMap() {

	}

	@Override
	public boolean connect() {
		boolean value = webThesaurus.connect();
		return value;
	}

	@Override
	public void destroy() {
		webThesaurus.destroy();
	}

	@Override
	public Long getTermCount(String key) {
		return webThesaurus.getTermCount(key);
	}

	@Override
	public Long getContextsCount(String key) {
		return webThesaurus.getContextsCount(key);
	}

	@Override
	public Long getTermContextsCount(String key, String val) {
		return webThesaurus.getTermContextsCount(key, val);
	}

	@Override
	public Double getTermContextsScore(String key, String val) {
		return webThesaurus.getTermContextsScore(key, val);
	}

	@Override
	public Map<String, Double> getSimilarTerms(String key) {
		return getStringDoubleMap(webThesaurus.getSimilarTerms(key));
	}

	@Override
	public Map<String, Double> getSimilarTerms(String key, int numberOfEntries) {
		return getStringDoubleMap(webThesaurus.getSimilarTerms(key, numberOfEntries));
	}

	@Override
	public Map<String, Double> getSimilarTerms(String key, double threshold) {
		return getStringDoubleMap(webThesaurus.getSimilarTerms(key, threshold));
	}

	@Override
	public Map<String, Double> getSimilarContexts(String values) {
		return getStringDoubleMap(webThesaurus.getSimilarContexts(values));
	}

	@Override
	public Map<String, Double> getSimilarContexts(String values,
			int numberOfEntries) {
		return getStringDoubleMap(webThesaurus.getSimilarContexts(values, numberOfEntries));
	}

	@Override
	public Map<String, Double> getSimilarContexts(String values,
			double threshold) {
		return getStringDoubleMap(webThesaurus.getSimilarContexts(values, threshold));
	}

	@Override
	public Map<String, Double> getTermContextsScores(String key) {
		return getStringDoubleMap(webThesaurus.getTermContextsScores(key));
	}

	@Override
	public Map<String, Double> getTermContextsScores(String key,
			int numberOfEntries) {
		return getStringDoubleMap(webThesaurus.getTermContextsScores(key, numberOfEntries));
	}

	@Override
	public Map<String, Double> getTermContextsScores(String key,
			double threshold) {
		return getStringDoubleMap(webThesaurus.getTermContextsScores(key, threshold));
	}

	public Map<String, Double> getStringDoubleMap(List<Pair<String, Double> > set) {
		Map<String, Double> map = new LinkedHashMap<String, Double>();
		if (set == null)
			return map;
		for (Pair<String, Double> p : set) {
			map.put(p.first, p.second);
		}
		return map;
	}

	@Override
	public Map<Integer, List<String>> getSenses(String key,String type) {
		Map<Integer, List<String>> results = new LinkedHashMap<Integer, List<String>>();

		List<Sense> result = webThesaurus.getSenses(key,type);
		if (result == null)
			return new LinkedHashMap<Integer, List<String>>();
		for (Sense s: result) {
			results.put(Integer.parseInt(s.getCui()), s.getSenses());
		}
		return results;
	}

	@Override
	public Map<Integer, Map<String, Double>> getIsas(String key,String type) {
		Map<Integer, Map<String, Double>> clusterIsas = new LinkedHashMap<Integer, Map<String, Double>>();
		List<Sense> result = webThesaurus.getSenses(key,type);
		if (result == null)
			return clusterIsas;
		for (Sense s: result) {
			Map<String,Double> isaMap = new LinkedHashMap<String, Double>();
			for (String isa : s.getIsas()) {
				String[] i = isa.split(":");
				isaMap.put(i[0], Double.parseDouble(i[1]));
			}
			clusterIsas.put(Integer.parseInt(s.getCui()), isaMap);
		}
		return clusterIsas;
	}

	@Override
	public Map<Integer, Map<String, Double>> getSenseCUIs(String key,String type) {
		throw new IllegalArgumentException("Method not implemented");
	}

	@Override
	public Double getSimilarTermScore(String t1, String t2) {
		return webThesaurus.getSimilarTermScore(t1, t2);

	}

	@Override
	public Map<String, Double> getContextsTermScores(String key) {
		return getStringDoubleMap(webThesaurus.getContextsTermScores(key));
	}

	@Override
	public Map<String, Double> getContextsTermScores(String key,
			int numberOfEntries) {
		return getStringDoubleMap(webThesaurus.getContextsTermScores(key, numberOfEntries));
	}

	@Override
	public Map<String, Double> getContextsTermScores(String key,
			double threshold) {
		return getStringDoubleMap(webThesaurus.getContextsTermScores(key,threshold));
	}

	@Override
	public Map<Integer, List<String>> getSenses(String key) {
		return getSenses(key, getStandardSense());
	}

	@Override
	public Map<Integer, Map<String, Double>> getIsas(String key) {
		return getIsas(key, getStandardSense());
	}

	@Override
	public Map<Integer, Map<String, Double>> getSenseCUIs(String key) {
		return getSenseCUIs(key, getStandardSense());
	}

	@Override
	public String[] getSensesTypes() {
		return webThesaurus.getSensesTypes();
	}

	@Override
	public String getStandardSense() {
		return webThesaurus.getStandardSense();
	}

	@Override
	public boolean isTermContained(String key) {
		return this.webThesaurus.isTermContained(key);
	}

	@Override
	public String[] isTermContained(String... key) {
		return this.webThesaurus.isTermContained(key);
	}
	
	@Override
	public boolean isContextsContained(String key) {
		return this.webThesaurus.isContextsContained(key);
	}

	@Override
	public String[] isContextsContained(String... key) {
		return this.webThesaurus.isContextsContained(key);
	}

}
