package org.jobimtext.api.struct;

import java.util.ArrayList;
import java.util.List;

import org.apache.pig.impl.util.Pair;
import org.jobimtext.api.webthesaurus.WebThesaurus;

/**
 * The WebThesaurusDatastructure can access a JoBimViz instance for retrieving data and performing sentence holing operations
 * @author Eugen Ruppert
 * 
 */
public class WebThesaurusDatastructure implements
		IThesaurusDatastructure<String, String>, WebThesaurusInterface<String, String>{

	WebThesaurus webThesaurus = new WebThesaurus();
	
	/**
	 * Constructs a new WebThesaurusDatastructure from a configuration file
	 * @param configurationFile path to the configuration file
	 */
	public WebThesaurusDatastructure(String configurationFile) {
			webThesaurus.setWebConfigurationFile(configurationFile);
	}

	/**
	 * Constructs a new WebThesaurusDatastructure
	 */
	public WebThesaurusDatastructure() {

	}

	@Override
	public boolean connect() {
		boolean value = webThesaurus.connect();
		return value;
	}

	@Override
	public void destroy() {
		webThesaurus.destroy();
	}
	
	public List<Pair<String, List<String> >> getSentenceHoling(String sentence) {
		return webThesaurus.getSentenceHoling(sentence);
	}

	@Override
	public List<Order2> getSimilarTerms(String key) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getSimilarTerms(key);
			return fillExpansions(set);
		} catch (Exception e) {
			e.printStackTrace();
			return new ArrayList<Order2>();
		}
	}
	@Override
	public List<Order2> getSimilarTerms(String key, int numberOfEntries) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getSimilarTerms(key, numberOfEntries);
			return fillExpansions(set);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	@Override
	public List<Order2> getSimilarTerms(String key, double threshold) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getSimilarTerms(key, threshold);
			return fillExpansions(set);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	private List<Order2> fillExpansions(List<Pair<String,Double>> set) {
		List<Order2> list = new ArrayList<Order2>();
		for (Pair<String, Double> p : set) {			
			list.add(new Order2(p.first, p.second));
		}
		return list;
	}
	
	private List<Order1> fillKeyValuesScores(List<Pair<String,Double>> set) {

		List<Order1> list = new ArrayList<Order1>();
		for (Pair<String, Double> p : set) {			
			list.add(new Order1(p.first, p.second));
		}
		return list;
	}
	
	
	@Override
	public Double getSimilarTermScore(String t1, String t2) {
		return webThesaurus.getSimilarTermScore(t1, t2);
	}

	

	@Override
	public List<Order2> getSimilarContexts(String value) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getSimilarContexts(value);
			return fillExpansions(set);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	@Override
	public List<Order2> getSimilarContexts(String value, int numberOfEntries) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getSimilarContexts(value, numberOfEntries);
			return fillExpansions(set);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	@Override
	public List<Order2> getSimilarContexts(String value, double threshold) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getSimilarContexts(value, threshold);
			return fillExpansions(set);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	@Override
	public Long getTermCount(String key) {
		return webThesaurus.getTermCount(key);
	}

	@Override
	public Long getContextsCount(String key) {
		return webThesaurus.getContextsCount(key);
	}

	@Override
	public Long getTermContextsCount(String key, String val) {
		return webThesaurus.getTermContextsCount(key, val);
	}

	@Override
	public Double getTermContextsScore(String key, String val) {
		return webThesaurus.getTermContextsScore(key, val);
	}

	@Override
	public List<Order1> getTermContextsScores(String key) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getTermContextsScores(key);
			return fillKeyValuesScores(set);
		} catch (Exception e) {
			e.printStackTrace();
			return new ArrayList<Order1>();
		}
	}

	@Override
	public List<Order1> getTermContextsScores(String key, int numberOfEntries) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getTermContextsScores(key, numberOfEntries);
			return fillKeyValuesScores(set);
		} catch (Exception e) {
			e.printStackTrace();
			return new ArrayList<Order1>();
		}
	}

	@Override
	public List<Order1> getTermContextsScores(String key, double threshold) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getTermContextsScores(key, threshold);
			return fillKeyValuesScores(set);
		} catch (Exception e) {
			e.printStackTrace();
			return new ArrayList<Order1>();
		}
	}

	@Override
	public List<Order1> getContextsTermScores(String key) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getContextsTermScores(key);
			return fillKeyValuesScores(set);
		} catch (Exception e) {
			throw new IllegalStateException("No contextTerm scores available for the current model!");
		}
	}

	@Override
	public List<Order1> getContextsTermScores(String key, int numberOfEntries) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getContextsTermScores(key, numberOfEntries);
			return fillKeyValuesScores(set);
		} catch (Exception e) {
			throw new IllegalStateException("No contextTerm scores available for the current model!");
		}
	}

	@Override
	public List<Order1> getContextsTermScores(String key, double threshold) {
		try {
			List<Pair<String, Double>> set = webThesaurus.getContextsTermScores(key, threshold);
			return fillKeyValuesScores(set);
		} catch (Exception e) {
			throw new IllegalStateException("No contextTerm scores available for the current model!");
		}
	}

	@Override
	public List<Sense> getSenses(String key) {
		return webThesaurus.getSenses(key);
	}

	@Override
	public List<Sense> getIsas(String key) {
		return webThesaurus.getIsas(key);
	}

	@Override
	public List<Sense> getSenseCUIs(String key) {
		return webThesaurus.getSenseCUIs(key);
	}

	@Override
	public List<Sense> getSenses(String key, String type) {
		return webThesaurus.getSenses(key, type);
	}

	@Override
	public List<Sense> getIsas(String key, String type) {
		return webThesaurus.getIsas(key, type);
	}

	@Override
	public List<Sense> getSenseCUIs(String key, String type) {
		return webThesaurus.getSenseCUIs(key, type);
	}

	@Override
	public String[] getSensesTypes() {
		return webThesaurus.getSensesTypes();
	}

	@Override
	public String getStandardSense() {
		return webThesaurus.getStandardSense();
	}

	@Override
	public boolean isTermContained(String key) {
		return webThesaurus.isTermContained(key);
	}

	@Override
	public String[] isTermContained(String... key) {
		return webThesaurus.isTermContained(key);
	}

	@Override
	public boolean isContextsContained(String key) {
		return webThesaurus.isContextsContained(key);
	}

	@Override
	public String[] isContextsContained(String... key) {
		return webThesaurus.isContextsContained(key);
	}

}
