/*******************************************************************************
* Copyright 2012

* 
* FG Language Technologie
* Technische Universitaet Darmstadt
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
******************************************************************************/
package org.jobimtext.api.webthesaurus;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URL;
import java.util.Map;

import javax.xml.bind.JAXBException;

import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.resource.ResourceSpecifier;
import org.apache.uima.resource.Resource_ImplBase;
import org.jobimtext.api.configuration.WebThesaurusConfiguration;

public class WebResource extends Resource_ImplBase {

	public static final String PARAM_XML_CONFIGURATION_FILE = "ServerConfigurationFile";
	public static final String PARAM_SORT_RESULTS = "SortResults";

	private File webConfigurationFile;
	private WebThesaurusConfiguration webConf;
	
	//@ConfigurationParameter(name = PARAM_SORT_RESULTS, mandatory = false)
	private Boolean sort = false;

	public WebResource() {

	}
	public WebResource(String xmlConf) {
		this(xmlConf,  true);
	}

	public WebResource(String xmlConf, boolean sort) {
		this.webConfigurationFile = new File(xmlConf);
		this.sort = sort;
	}
	
	
	@Override
	public boolean initialize(ResourceSpecifier aSpecifier, Map<String, Object> aAdditionalParams) throws ResourceInitializationException {
		boolean returnCode = super.initialize(aSpecifier, aAdditionalParams);
		returnCode &= connect();
		return returnCode;

	}

	public boolean connect() {		
		try {
			webConf = WebThesaurusConfiguration.getFromXmlFile(webConfigurationFile);
			HttpURLConnection conn;
			URI uri = new URI(this.getWebConfiguration().getProtocol(), null,
					this.getWebConfiguration().getServer(), this.getWebConfiguration().getPort(),
					this.getWebConfiguration().getAPIPath()+this.getWebConfiguration().getDataset()+"/jo/similar/1",
					null, null);
			URL url = new URL(uri.toASCIIString());
			conn = (HttpURLConnection) url.openConnection();
			conn.setRequestMethod("GET");			
			conn.getInputStream();
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	
	public boolean isSort() {
		return sort;
	}

	public void setSort(boolean sort) {
		this.sort = sort;
	}

	public File getDbConfigurationFile() {
		return webConfigurationFile;
	}

	public void setWebConfigurationFile(File configurationFile) {
		this.webConfigurationFile = configurationFile;
		try {
			WebThesaurusConfiguration.getFromXmlFile(webConfigurationFile);
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void setWebConfigurationFile(String configurationFile) {
		this.webConfigurationFile = new File(configurationFile);
	}

	public WebThesaurusConfiguration getWebConfiguration() {
		return webConf;
	}	

	@Override
	public void destroy() {
		super.destroy();
	}

	@Override
	protected void finalize() throws Throwable {
		destroy();
		super.finalize();
	}

}

