package org.jobimtext.contextualize;

import java.lang.reflect.InvocationTargetException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.xml.bind.JAXBException;

import org.apache.uima.UimaContext;
import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.fit.component.JCasAnnotator_ImplBase;
import org.apache.uima.fit.descriptor.ConfigurationParameter;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;
import org.apache.uima.jcas.cas.DoubleArray;
import org.apache.uima.jcas.cas.FSArray;
import org.apache.uima.jcas.tcas.Annotation;
import org.apache.uima.resource.ResourceInitializationException;
import org.jobimtext.api.struct.DatabaseThesaurusDatastructure;
import org.jobimtext.api.struct.IThesaurusDatastructure;
import org.jobimtext.api.struct.Order2;
import org.jobimtext.holing.extractor.JobimAnnotationExtractor;
import org.jobimtext.holing.extractor.JobimExtractorConfiguration;
import org.jobimtext.holing.type.JoBim;
import org.jobimtext.holing.type.Sentence;
import org.jobimtext.type.DistributionalExpansion;
import org.jobimtext.type.DistributionalExpansions;

public class RandomContextualizer extends JCasAnnotator_ImplBase {

	public static final String PARAM_API_CONFIG = "ApiConfig";
	public static final String PARAM_EXTRACTOR_CONFIG = "ExtractorConfig";
	public static final String PARAM_SENTENCE = "SentenceAnnotation";
	public static final String PARAM_NUMBER_OF_EXPANSIONS = "NumberOfExpansions";
	@ConfigurationParameter(name = PARAM_NUMBER_OF_EXPANSIONS)
	private int numberOfEntries;
	@ConfigurationParameter(name = PARAM_API_CONFIG)
	String apiConfig;
	@ConfigurationParameter(name = PARAM_EXTRACTOR_CONFIG)
	String extractorConfig;
	private JobimAnnotationExtractor extractor;
	@ConfigurationParameter(name = PARAM_SENTENCE, mandatory = false, description = "Specifies the name of the sentence anntation")
	private String sentenceAnnotationClassName = Sentence.class.getName();
	private Class<Annotation> sentenceAnnotationClass;
	private IThesaurusDatastructure<String, String> dt;

	@Override
	public void initialize(UimaContext context)
			throws ResourceInitializationException {
		super.initialize(context);
		try {
			extractor = JobimExtractorConfiguration.getExtractorFromXmlFile(extractorConfig);
			dt=  new DatabaseThesaurusDatastructure(apiConfig);
			dt.connect();
			sentenceAnnotationClass = (Class<Annotation>) Class
					.forName(sentenceAnnotationClassName);
			if (sentenceAnnotationClass != null) {
				if (sentenceAnnotationClassName.equals(sentenceAnnotationClass)) {
					sentenceAnnotationClassName = null;
				} else {
					sentenceAnnotationClass = (Class<Annotation>) Class
							.forName(sentenceAnnotationClassName);
				}
			}
		} catch (InstantiationException | IllegalAccessException
				| IllegalArgumentException | InvocationTargetException
				| NoSuchMethodException | SecurityException
				| ClassNotFoundException | JAXBException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void process(JCas jcas) throws AnalysisEngineProcessException {
		Set<Annotation> filled = new HashSet<Annotation>();
		for (Annotation s : JCasUtil.select(jcas, sentenceAnnotationClass)) {
			for (JoBim jb : JCasUtil.selectCovered(JoBim.class,s)){
				if (!filled.contains(jb.getKey())){
					String key = extractor.extractKey(jb);
					List<Order2> exps = dt.getSimilarTerms(key, numberOfEntries);
					DistributionalExpansions uexps = new DistributionalExpansions(jcas,jb.getKey().getBegin(),jb.getKey().getEnd());
					int i = 0;
					uexps.setEntries(new FSArray(jcas, exps.size()));
					for (Order2 o2 : exps){
						DistributionalExpansion e = new DistributionalExpansion(jcas, uexps.getBegin(), uexps.getEnd());
						e.setKey(o2.key);
						e.setScore(o2.score);
						DoubleArray arr = new DoubleArray(jcas, 2);
						arr.set(0, Math.random());
						arr.set(1, Math.random());
						
						e.setContextScores(arr);
						//e.setContextScore(Math.random());
						uexps.setEntries(i, e);
						i+=1;
					}
					uexps.addToIndexes();
					filled.add(jb.getKey());
				}
			}
		}
	}

}
