package org.jobimtext.hadoop.mapreducer;

import java.io.IOException;

import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reporter;
import org.jobimtext.util.StringUtils;

public abstract class SimCountMapper extends MapReduceBase implements
		Mapper<LongWritable, Text, Text, Text> {

	public abstract double getWeight(double elements);

	private boolean addFeatures = false;

	@Override
	public void configure(JobConf job) {
		super.configure(job);
		addFeatures = job.getBoolean("addFeatures", addFeatures);
	}

	public void map(LongWritable key, Text value,
			OutputCollector<Text, Text> output, Reporter reporter)
			throws IOException {

		// split line into tokens
		// the first is the feature, the rest are all
		// words having this feature

		String[] line = StringUtils.splitWithoutEmptyElements(value.toString(),
				'\t');
		int linesize = line.length;
		// set weight to one
		double weight = getWeight(linesize);
		String context = line[0];
		// outputValue is the weight and the feature
		Text outputValue;
		if (addFeatures) {
			outputValue = new Text(Double.toString(weight) + "\t" + context);
		} else {
			outputValue = new Text(Double.toString(weight));
		}

		// count all word pairs in that line
		// skip first entry as this is the feature
		for (int i = 1; i < linesize; i++) {

			for (int j = 1; j < linesize; j++) {

				/*
				 * output words in both directions
				 */
				output.collect(new Text(line[i] + "\t" + line[j]), outputValue);

			} // for (inner)

		} // for (outer)

		// } // if < threshold

	} // map()

} // class map

