package org.jobimtext.holing.segmenter;

import org.apache.uima.UimaContext;
import org.apache.uima.analysis_component.JCasAnnotator_ImplBase;
import org.apache.uima.fit.descriptor.ConfigurationParameter;
import org.apache.uima.jcas.JCas;
import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.util.Level;
import org.apache.uima.util.Logger;
import org.jobimtext.holing.type.Sentence;


public class SentencePerLineAnnotatorIndexed extends JCasAnnotator_ImplBase {
	

	public static final String PARAM_SENTENCE_FIELD_ID= "SentenceField";
	@ConfigurationParameter(name = PARAM_SENTENCE_FIELD_ID, mandatory = true)
	private int sentenceField = 1;

	public static final String PARAM_IDENTIFIER_FIELD_ID= "SentenceIdentifierField";
	@ConfigurationParameter(name = PARAM_IDENTIFIER_FIELD_ID, mandatory = true)
	private int identifierField = 0;
	
	private static Logger logger;
	private int maxField = 0;
	@Override
	public void initialize(UimaContext context) throws ResourceInitializationException {
		super.initialize(context);

		logger = context.getLogger();
		
		if (context.getConfigParameterValue(PARAM_SENTENCE_FIELD_ID) != null) {
			sentenceField = Integer.parseInt(context.getConfigParameterValue(PARAM_SENTENCE_FIELD_ID).toString());
		}
		if (context.getConfigParameterValue(PARAM_IDENTIFIER_FIELD_ID) != null) {
			identifierField = Integer.parseInt(context.getConfigParameterValue(PARAM_IDENTIFIER_FIELD_ID).toString());
		}
		
		maxField = Math.max(sentenceField, identifierField);
	
		if (sentenceField == identifierField) {
			throw new ResourceInitializationException("Same fields are used for sentence and sentenceID. Please ensure that they have distinct values!", null);
		}
	}




	public void process(JCas aJCas) {
		String aText = aJCas.getDocumentText();
		int length = aText.length();
		int sentenceBegin = 0;
		int sentenceEnd = 0;
		String identifier = "";

		int currentField = 0;
		int lineOffset = 0;
		int lastTab = 0;
		
		for (int i = 0 ; i< length; i++) {
			if (aText.charAt(i) == '\t' || aText.charAt(i) == '\n') {
				if (currentField == sentenceField) {
					sentenceBegin = lastTab;
					sentenceEnd = i;
				} else if (currentField == identifierField) {
					identifier = aText.substring(lastTab, i);
				}
		        lastTab = i+1;
			} 
			if (aText.charAt(i) == '\t') {
				currentField++;
			}
			if (aText.charAt(i) == '\n') {
	        	if (currentField < maxField) {
	        		logger.log(Level.SEVERE, "Line does not contain all neccessary fields! Affected line: '" + 
	        				aText.substring(lineOffset, i) + "'\n" +
	        				"Skipping line...");
	        	} else {
	        		createSentence(aJCas, sentenceBegin, sentenceEnd, identifier);
	        	}
	        	currentField = 0;
		        lineOffset = i+1;
		        sentenceBegin = i+1;
		        sentenceEnd = i+1;
		        identifier = null;
	        }
		}
	}

	private void createSentence(JCas aJCas, int begin, int end, String identifier) {
		if (isEmpty(begin, end)) return;
		Sentence sentence = new Sentence(aJCas, begin, end);
		sentence.setId(identifier);
		sentence.addToIndexes();
	}

	public boolean isEmpty(int aBegin, int aEnd) {
		return aBegin >= aEnd;
	}

}
