/*******************************************************************************
* Copyright 2012
* Copyright (c) 2012 IBM Corp.
* 
* and
* 
* FG Language Technologie
* Technische Universitaet Darmstadt
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
******************************************************************************/
package org.jobimtext.sense;

import java.io.File;
import java.io.IOException;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;

/**
 * The main entry point to compute sense clusters using
 * the Chinese Whispers algorithm of Chris Biemann.
 * 
 * @param args - command line arguments
 */
public class ComputeSenseClusters {

  public static void main(String[] args) {
	  System.out.println(Integer.MAX_VALUE);
	  
    CommandLine cmd = parseParameters(args);
    if (args.length < 1) {
		  printErrorMessage();
	  }    
    // generate the node and edge files
    NodeEdgeFileWriter nefw = new NodeEdgeFileWriter();
    String tempFolder = cmd.getOptionValue("tf");
	if (tempFolder == null) {
		tempFolder = "./temp";
	}
    new File(tempFolder).mkdir();
    try {
      nefw.generateFiles(
    	  cmd.getOptionValue("i"),
          tempFolder, 
          cmd.getOptionValue("wl"), 
          cmd.getOptionValue("el"));
    } catch (IOException e) {
      e.printStackTrace();
      System.exit(1);
    }
        
    // run the cwd algorithm
    RunCWD cwd = new RunCWD();
    cwd.run(cmd);      
  }

  private static void printErrorMessage() {
	  System.err.println("Insufficient parameters specified!");
	  System.err.println("Usage: java -cp CLASSPATH org.jobimtext.sense.ComputeSenseClusters -i INPUT_FILE -o OUTPUT_FILE -N NUMBER -n NUMBER");
	  HelpFormatter formatter = new HelpFormatter();
      formatter.printHelp( "ComputeSenseClusters", createOptions());
      System.exit(1);
	
}

private static CommandLine parseParameters(String[] args) {
    Options options = createOptions();
    CommandLineParser parser = new PosixParser();
    CommandLine cmd = null;
    try {
      cmd = parser.parse(options, args);
    } catch (ParseException e) {
    	printErrorMessage();
    }
 
    return cmd;
  }
  
  @SuppressWarnings("static-access")
private static Options createOptions() {
    Options options = new Options();
    options.addOption(OptionBuilder.withLongOpt("output").isRequired().hasArg().withDescription("*Required: output file for the sense clusters").create("o"));
    options.addOption(OptionBuilder.withLongOpt("input").isRequired().hasArg().withDescription("*Required: input simsort file (DT) from jobimtext pipeline").create("i"));
    options.addOption(OptionBuilder.withLongOpt("N").isRequired().hasArg().withDescription("*Required: entry creation top N filter").create("N"));
    options.addOption(OptionBuilder.withLongOpt("n").isRequired().hasArg().withDescription("*Required: edge top N within entry").create("n"));
   
    options.addOption("a", "algorithm", true, "sets algorithm options");
    options.addOption("h", "help", true, "displays help message");
    options.addOption("wl", "words", true, "location of word list file");
    options.addOption("el", "edges", true, "location of edge list file");    
    options.addOption("d", "iterations", true, "number of iterations");
    options.addOption("dt", "dt", true, "on-disk edge threshold");
    options.addOption("t", "t", true, "edge threshold within entries");
    options.addOption("T", "T", true, "edge threshold for entry creation");
    options.addOption("mc", "mc", true, "minimal cluster size");
    options.addOption("mr", "mr", true, "maximal cluster rank");
    options.addOption("ms", "ms", true, "minimal similarity score");
    options.addOption("S", "S", true, "input will not be sorted and renumbered");
    options.addOption("from", "from", true, "the word number to start from");
    options.addOption("to", "t", true, "the word number to end with");
    options.addOption("tg", "tg", true, "file containing target words");
    options.addOption("tf", "temp-out", true, "output folder to store temporary files");
    return options;
  }
  
}
